import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';

@immutable
abstract class HomeState extends Equatable {
  HomeState([List props = const []]) : super();

  @override
  // TODO: implement props
  List<Object> get props => [];
}

class InitialHomeState extends HomeState {}

class HomeLoading extends HomeState {
  @override
  String toString() => 'ListServiceLoading';
}

class GetHomeSuccess extends HomeState {
  @override
  String toString() {
    return 'GetHomeSuccess';
  }
}

class HomeFailure extends HomeState {
  final String error;

  HomeFailure(this.error);

  @override
  String toString() => 'ListServiceFailure { error: $error }';
}

class HomeDate extends HomeState {
  @override
  String toString() {
    // TODO: implement toString
    return 'ListServiceDate{}';
  }
}

class GetListTaskEmpty extends HomeState {
  @override
  String toString() {
    // TODO: implement toString
    return 'GetListTaskEmpty{}';
  }
}

class ChangeCurrentTabSuccess extends HomeState {}
